package psw2.JIM;

import java.awt.*;

public class JIMPolygon extends Polygon {
  static final int PLAIN_VERTEX =0x00000000;
  static final int SHARP_VERTEX =0x00000001;
    
  public int vert_prop[] = new int[3];  //the properties for the vertices
    
  public JIMPolygon(int xpoints[], int ypoints[], int vert_prop[], int npoints) {
    //addpts is 0 if Array is already closed, 1 otherwise (one Point is to add)
    int addpts=(((xpoints[npoints-1]==xpoints[0])&&(ypoints[npoints-1]==ypoints[0]))?0:1);
  
    this.npoints = npoints+addpts;
    this.xpoints = new int[npoints+addpts];
    this.ypoints = new int[npoints+addpts];
    this.vert_prop=new int[npoints+addpts];
    System.arraycopy(xpoints, 0, this.xpoints, 0, npoints);
    System.arraycopy(ypoints, 0, this.ypoints, 0, npoints); 

    if (vert_prop!=null) {  //copy and fill rest with 0's
      System.arraycopy(vert_prop  , 0, this.vert_prop  , 0, vert_prop.length);  
      for (int i=vert_prop.length;i<npoints+addpts;i++) this.vert_prop[i]=PLAIN_VERTEX;   
    } else {  
      //init vert_props with PLAIN_VERTEX
      for (int i=0;i<npoints+addpts;i++) this.vert_prop[i]=PLAIN_VERTEX;
    }   

    if (addpts!=0) //so polygon was not closed, about to close it
    {
      this.xpoints[npoints]=xpoints[0];
      this.ypoints[npoints]=ypoints[0];
    }
  }

  
  public JIMPolygon(int xpoints[], int ypoints[], int vert_prop[]) {
    this(xpoints,ypoints,vert_prop,xpoints.length);
  }

  
  public JIMPolygon(int xpoints[], int ypoints[],int vert_prop) {
    this(xpoints,ypoints);
    
    for (int i=0;i<npoints;i++)
      this.vert_prop[i]=vert_prop;
    }

    public JIMPolygon(JIMPolygon source) {
      this(source.xpoints,source.ypoints,source.vert_prop,source.npoints);
      this.bounds=source.bounds;
    }
        
    public JIMPolygon(int xpoints[], int ypoints[]) {
      this(xpoints,ypoints,null,xpoints.length);
    }

    public JIMPolygon(int xpoints[], int ypoints[],Rectangle bounds) {
      this(xpoints,ypoints,null,xpoints.length);
      this.bounds=bounds;
    }
              
    //because updateBounds from java.awt.Polygon is only
    //visible in java.awt, and updateBounds is needed for
    //addPoint
    void accessibleupdateBounds(int x, int y) {
      if (x < bounds.x) {
        bounds.width = bounds.width + (bounds.x - x);
        bounds.x = x;
      } else {
        bounds.width = Math.max(bounds.width, x - bounds.x);
    }

    if (y < bounds.y) {
      bounds.height = bounds.height + (bounds.y - y);
      bounds.y = y;
    } else {
      bounds.height = Math.max(bounds.height, y - bounds.y);
    }
  } 
    
  public void addPoint(int x, int y, int vert_prop) { 
    int tmp[];
  
     npoints++; 
    //handle xpoints
    tmp = new int[npoints];
    System.arraycopy(xpoints, 0, tmp, 0, npoints-2);
    xpoints = tmp;
    xpoints[npoints-1]=xpoints[0];
    xpoints[npoints-2]=x;

    //handle ypoints
    tmp = new int[npoints];
    System.arraycopy(ypoints, 0, tmp, 0, npoints-2);
    ypoints = tmp;
    ypoints[npoints-1]=ypoints[0];
    ypoints[npoints-2]=y;
        
    //handle vert_prop;
    tmp = new int[npoints];
    System.arraycopy(this.vert_prop, 0, tmp, 0, npoints-1);
    this.vert_prop = tmp;
    this.vert_prop[npoints-1]=this.vert_prop[npoints-2];
    this.vert_prop[npoints-2]=vert_prop;        
  
    //update the bounds (if there is already a bounds object)
    if (bounds != null) {
      accessibleupdateBounds(x, y);
    }
  }
    
    
  //returns   0   if (x,y) is ON the Polygon
  //    <0  if (x,y) is INSIDE the Polygon
  //    >0  if (x,y) is OUTSIDE of the Polygon
  public int dist(int x, int y) {
    int xd,yd;  //direction of left-rotated normal vertex of
    //actual vertex of polygon
    int d;    //distance from origin to the vertex
    int i,res=-1;
      
    for (i=0;(i<npoints-1) && (res<0);i++) {
      yd=-(xpoints[i+1]-xpoints[i]);
      xd=ypoints[i+1]-ypoints[i];
      d=xpoints[i]*xd+ypoints[i]*yd;
      res=(x*xd+y*yd)-d;        
    }
    return res; 
  }
    
  public Object[] intersects(JIMPolygon p) {
    java.util.Vector ret1=new java.util.Vector(),ret2=new java.util.Vector();
    int i,j,gbx,gby,gdx,gdy,hbx,hby,hdx,hdy;
    double s,t,quot;
      
    for (i=0;i<npoints-1;i++)
    for (j=0;j<p.npoints-1;j++) { 
      gbx=xpoints[i];     
      gby=ypoints[i];
      gdx=xpoints[i+1]-xpoints[i];  
      gdy=ypoints[i+1]-ypoints[i];
      hbx=p.xpoints[j];   
      hby=p.ypoints[j];
      hdx=p.xpoints[j+1]-p.xpoints[j];
      hdy=p.ypoints[j+1]-p.ypoints[j];
      quot=(-gdy*hdx+gdx*hdy);
      if (quot!=0) {
        quot=1/quot;
        s=(gby*hdx-hby*hdx-gbx*hdy+hbx*hdy)*quot;
        if ((s>=0) && (s<=1)) {
          t=(gby*gdx-gbx*gdy+gdy*hbx-gdx*hby)*quot;
          if ((t>=0) && (t<=1)) {
            ret1.addElement(new Integer(i));
            ret2.addElement(new Integer(j));
          }
        }
      }
    }
    return (ret1.size()>0) ? new Object[] {ret1,ret2,null} : null ;
  }
    
 
  public void setSize(int width, int height) {
    float factx,facty;
    if (bounds==null) getBounds();
    if ((width>0) && (height>0) && (bounds.width>0) && (bounds.height>0)) {
      factx=((float)width)/bounds.width;
      facty=((float)height)/bounds.height;
      for (int i=0;i<npoints;i++) {
        xpoints[i]=(int)(xpoints[i]*factx);
        ypoints[i]=(int)(ypoints[i]*facty);
      }
      bounds=null;
    }
  }
   
 
  public String toString() {
    String ret;
    int i;
    ret=" ("+npoints;
    for (i=0;i<npoints;i++) {
      ret=ret+"("+xpoints[i]+","+ypoints[i]+","+vert_prop[i]+"),";
    }
    ret=ret+")";
    return ret;
  }
    
 
  public void paint(java.awt.Graphics g) {
    java.awt.Color oldcol=g.getColor();
    g.setColor(java.awt.Color.white);
    for (int i=1;i<npoints;i++) {
      g.drawLine(xpoints[i-1],ypoints[i-1],xpoints[i],ypoints[i]);
    }
    g.setColor(oldcol);
  }
    
    
  public void mirror(float yaxis) {
    int last=xpoints.length-1;//the last vertex to flip
    int swaps=(xpoints.length>>1); //the number of swaps to do;
    int swaphelp;
    int shiftvalue=(int)(yaxis*2); //the fixed amount to add when mirroring
    for (int i=0;i<swaps;i++) {
      swaphelp=xpoints[i]+shiftvalue-2*xpoints[i];
      xpoints[i]=xpoints[last-i]+shiftvalue-2*xpoints[last-i];
      xpoints[last-i]=swaphelp;

      swaphelp=ypoints[i];
      ypoints[i]=ypoints[last-i];
      ypoints[last-i]=swaphelp;
    
      swaphelp=vert_prop[i];
      vert_prop[i]=vert_prop[last-i];
      vert_prop[last-i]=swaphelp;
    }
    if ((xpoints.length&1)==1)//odd number of vertices
    {//so we forgot to mirror the vertex in the middle
      int i=swaps;
      xpoints[i]=xpoints[i]+shiftvalue-2*xpoints[i];
    }  
  }
}